/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     dk40demo.c
* Function:   Example for hardware API calls on the DK40
*
*******************************************************************************
*
* $Header: DK40DEMO.C, 2, 31.03.2004 11:15:36, Andre Pribil$
*
******************************************************************************/

/***************************************************************************/
// Includes
/***************************************************************************/
#include <stdio.h>
#include <dos.h>
#include "clib.h"

/***************************************************************************/
// Variables
/***************************************************************************/
unsigned wPio, wInp, wOut;
unsigned pins;

/***************************************************************************/
// Init dk40
/***************************************************************************/
void init_dk40 (unsigned mask)
{
  unsigned char mode;

  // Enable bus mask, 7 bit outputs are on databus, 1 bit output is PIO
  pfe_enable_bus(mask & 0x7F, 0);        // disable ALE

  // Enable programmable chip selects
  pfe_enable_pcs(6);

  // Enable DK40 pin 7 as PIO
  if (mask&0x80)
  {
    // Pin 7 is output (low)
    mode = PIO_O0;
  }
  else
  {
    // Pin 7 is input pulldown
    mode = PIO_IPD;
  }

  // Enable programmable PIO13 / DK40 Pin7
  pfe_enable_pio(13, mode);         // output (Low) or input(pulldown)

  // Enable programmable PIO3 / DK40 LED1
  pfe_enable_pio(3, PIO_O1);        // output (High)
  pfe_get_pio_state(&wPio, &wInp, &wOut);

  // Read pio outputs init state
  pins = hal_read_pios (0xFFFF, 0x0000);   // wAND=0xFFFF, wXOR=0x0000
}

/***************************************************************************/
// Read dk40 inputs (and outputs)
/***************************************************************************/
unsigned char read_dk40_io (void)
{
  unsigned char value;

  // read data bus, E0.0-E0.7 on databus
  value = (unsigned char) hal_read_bus(0x600, 0xFFFF, 0x0000); // address = 600h,
                                                               // wAND=0xFFFF,
                                                               // wXOR=0x0000
  return value;
}

/***************************************************************************/
// Write dk40 outputs
/***************************************************************************/
void write_dk40_io (unsigned char value)
{
  // write data bus, A0.0-A0.6 on databus
  hal_write_bus(0x600, value, 0x007F, 0x0000); // address = 600h
                                               // wAND=only A0.0-A0.6
                                               // wXOR=0x0000

  // write pio pins, A0.7 is PIO13
  if (value&0x80)
  {
    hal_write_pio(13, 1);
  }
  else
  {
    hal_write_pio(13, 0);
  }
}

/***************************************************************************/
// Set dk40 led1
/***************************************************************************/
void set_dk40_led (unsigned char mode)
{
  hal_write_pio(3, mode);
}


/***************************************************************************/
// Get dk40 led1
/***************************************************************************/
int get_dk40_led(void)
{
  return hal_read_pio(3);
}



/***************************************************************************/
// Main
/***************************************************************************/
void main (void)
{
  unsigned char read_state, write_state;
  unsigned i;

  // EA0.7-EA0.4 outputs, EA0.3-EA0.0 inputs
  init_dk40(0xF0);

  // Display init state
  printf("\r\nwPIO: %04Xh, wINP: %04Xh, WOUT: %04Xh, pins: %04X",
         wPio,wInp,wOut,pins);


  write_state = 0x10;

  // run for 1 min
  for (i=0; i<60; i++)
  {
     // Write DK40 outputs
     write_dk40_io(write_state);

     // Read DK40 inputs
     read_state = read_dk40_io();

     // Display state
     printf("\r\nRead: %02Xh, Write: %02Xh",read_state,write_state);

     // Run light on A0.4-A0.7
     write_state<<=1;
     if (write_state==0x00)
     {
       write_state=0x10;
     }

     // Toogle LED
     set_dk40_led(!get_dk40_led());

     // Wait 1 sec
     RTX_Sleep_Time(1000);
  }

  // Clear outputs
  set_dk40_led(0);
  write_dk40_io(0);

  printf("\r\ndone");
}

