/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     msg.c
* Function:   This program demonstrates the usage of the RTOS-Interface of the
*             IPC@CHIP. Two tasks will be created, which both are waiting in FIFO
*             order for a message. If one a the tasks gets a message, it will
*             print it to stdout.
*             The user can send messages by pressing Return.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\rtos\MESSAGE\MSG.C, 4, 31.03.2004 11:15:50, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <conio.h>
#include "clib.h"

/******************************************************************************
* Constants
******************************************************************************/
#define TASK_STACKSIZE  1024   /*WORDS => 2048 Bytes, was 1024*/

/******************************************************************************
* Prototypes
******************************************************************************/
void huge task1(void);
void huge task2(void);

/******************************************************************************
* Global variables
******************************************************************************/
unsigned int task1_stack[TASK_STACKSIZE];
int task1ID;

TaskDefBlock  task1defblock =
{
  task1,
  {'T','A','S','1'},             // a name: 4 chars
  &task1_stack[TASK_STACKSIZE],  // top of stack
  TASK_STACKSIZE*sizeof(int),   // size of stack
  0,                             // attributes, not supported
  26,                            // priority 3(high) ... 127(low)
  0,                             // time slice (if any), not needed
  0,0,0,0                        // not supported
};

unsigned int task2_stack[TASK_STACKSIZE];
int task2ID;

TaskDefBlock  task2defblock =
{
  task2,
  {'T','A','S','2'},             // a name: 4 chars
  &task2_stack[TASK_STACKSIZE],  // top of stack
  TASK_STACKSIZE*sizeof(int),   // size of stack
  0,                             // attributes, not supported
  27,                            // priority 3(high) ... 127(low)
  0,                             // time slice
  0,0,0,0                        // not supported
};

int   print_semaphoreID; /*semaphoreID*/
long  semaphore_timout = 0L;
char  print_sem[4] = {'P','S','E','1'};
int   task1run;
int   task2run;

//buffer for task1
char msg_1[13];
//buffer for task2
char msg_2[13];

RTX_Msg create_msg =  {
                         -1,       //id, filled by rtx_create_msg
                         {'M','S','G','0'}, //name
                         2,        //mailbox 0 depth 2
                         0,
                         0,
                         0
                      };

//task 1
RTX_Wait_Msg wait_msg_T1 = { -1,     //id, filled later
                            0,      //highest prio
                            msg_1,  //msg buffer
                             10000L //wait max 10000 ms for the message
                            };

//task 2
RTX_Wait_Msg wait_msg_T2 = { -1,       //id filled later
                              0,       //highest prio
                              msg_2,   //msg buffer
                              10000L   //wait max 10000 ms for the event
                            };



/******************************************************************************
* print_msg_message()
*
* This function will be called fronm both tasks if the main loop signals the event.
* The function is protected with a resource semaphore to prevent being interrupted.
******************************************************************************/
void print_msg_message(char const * msg, int count)
{
  int result=0;
  long timeout;
  //reserve the resource, wait forever
  timeout=0L;
  result =  RTX_Reserve_Sem(print_semaphoreID,&timeout);
  if(result==0)
  {
    printf("\r\nTaskID %d Recvd. Msg.: %s, count %d",RTX_Get_TaskID(),msg, count);
  }
  RTX_Release_Sem(print_semaphoreID);//release the resource
}



/******************************************************************************
* task1()
******************************************************************************/
void huge task1(void)
{

   int result;
   static int count=0;
   static int timeouts=0;
   static int errs=0;

   task1run=1;

   while(task1run)
   {
      result=RTX_Wait_For_Msg(&wait_msg_T1);
      if(result==0)
      {
        print_msg_message(wait_msg_T1.msg,++count);
      }
      else
       if(result==-27)  //see api errocodes at html doc
       {
          print_msg_message("Timeout",++timeouts);
       }
       else
       {
          printf("\r\nResult %d ID %d",result,wait_msg_T1.msgID);
          print_msg_message("Other error",++errs);
       }
   }

    /*
      You should not delete a task, which is waiting, or is in about
      a wait, for a resource or counting semaphore.
      It should be safe to delete the task by itself after the
      while loop.
    */

   printf("\r\nRemoving task1");
   result = RTX_Delete_Task(RTX_Get_TaskID());
   if(result!=0)
   {
      printf(" failed, %d  %d\r\n",result,task1ID);
   }
}



/******************************************************************************
* task2()
******************************************************************************/
void huge task2(void)
{
   int result;
   static int count=0;
   static int timeouts=0;
   static int errs=0;

   task2run=1;
   while(task2run)
   {

      result=RTX_Wait_For_Msg(&wait_msg_T2);
      if(result==0)
      {
        print_msg_message(wait_msg_T2.msg,++count);
      }
      else
       if(result==-27)  //see api errocodes at html doc
       {
          print_msg_message("Timeout",++timeouts);
       }
       else
       {
          printf("\r\nResult %d ID %d",result,wait_msg_T2.msgID);
          print_msg_message("Other error",++errs);
       }
   }

   /*
      You should not delete a task, which is waiting, or is in about
      a wait, for a resource or counting semaphore.
      It should be safe to delete the task by itself after the
      while loop.
   */

   printf("\r\nRemoving task2");
   result = RTX_Delete_Task(RTX_Get_TaskID());
   if(result!=0)
   {
      printf(" failed, %d\r\n",result);
   }
}

/******************************************************************************
* main()
******************************************************************************/
int main(void)
{
   unsigned int ID;
   int result;
   int key=0;
   char msg_sendbuf[13];
   char msg_rcvbuf[13];
   char msgname[5]={'M','S','G','0',0};
   int i;

   //switch stdio to our own program
   BIOS_Set_Focus( FOCUS_APPLICATION );

   printf("\r\nRTOS API  message exchange example\r\n");

   //create a resource semaphore for function print_task_msg
   result = RTX_Create_Sem(&print_semaphoreID,print_sem,-1);
   if(result!=0)
   {
     printf("\r\nCreating the semaphore failed %d, exit program\r\n",result);
     return 0;
   }

   //create a message_exchange
   result = RTX_Create_Msg(&create_msg);
   if(result!=0)
   {
     printf("\r\nCreating the message exchange failed %d, exit program\r\n",result);
     return 0;
   }
   else
   {
      printf("\r\nMessage exchange created, ID: %d",create_msg.msgID);
   }


   ////Note: this is only for testing : find the message exchange
   printf("\r\nTest: find the exchange by name %s",msgname);
   result = RTX_Find_Msg(&ID, msgname);
   printf(" Result %d   ID %d ",result,ID);

   //Note: this is only for testing : test get a message
   //init message_sendbuffer
   for(i=0;i<12;i++)
   {
     msg_sendbuf[i]='Z'-1;
   }
   msg_sendbuf[12]=0;
   //first send a message
   result = RTX_Send_Msg(create_msg.msgID,0,msg_sendbuf); //highest prio
   if(result==0)
   {

       result=RTX_Get_Msg(create_msg.msgID,msg_rcvbuf);
       if(result==0)
       {
         printf("\r\nGet message: ");
         for(i=0;i<12;i++)printf("%c",msg_rcvbuf[i]);
       }
       else
       {
          printf("\r\nNo message available Result %d",result);
       }
   }



   //complete the RTX_Wait_Msg structure for each task, add the Msg exchange ID
   wait_msg_T1.msgID = wait_msg_T2.msgID = create_msg.msgID;


   //create and start task1
   result = RTX_Create_Task(&task1ID , &task1defblock);
   if(result!=0)
   {
     printf("\r\nCreating TASK_1 failed %d, exit program\r\n",result);
     //delete print_semaphore
     RTX_Delete_Sem(print_semaphoreID);
     return 0;
   }

   //create and start task2
   result = RTX_Create_Task(&task2ID , &task2defblock);
   if(result!=0)
   {
     printf("\r\nCreating TASK_2 failed %d, exit program\r\n",result);
     //delete task1
     RTX_Delete_Task(task1ID);
     //delete print_semaphore
     RTX_Delete_Sem(print_semaphoreID);
     return 0;
   }


   //init message_sendbuffer
   for(i=0;i<12;i++)
   {
     msg_sendbuf[i]='A'-1;
   }
   msg_sendbuf[12]=0;



   printf("\r\nPress Enter for sending a message, ESC exits program\r\n");
   while(key!=27)
   {
       key=getch();
       if(key==13)
       {
          //modify msg_buf
          //init msg_sendbuf
          if(msg_sendbuf[0]>='Z')
          {
             for(i=0;i<12;i++)
             {
               msg_sendbuf[i]='A'-1;
             }
          }
          else
          {
             for(i=0;i<12;i++)
             {
               msg_sendbuf[i]++;
             }
          }

          result = RTX_Send_Msg(create_msg.msgID,0,msg_sendbuf); //highest prio
          if(result!=0)
          {
            printf("\r\nSending message failed %d", result);
            break;
          }
       }

       RTX_Sleep_Time(100);
   }

   printf("\r\nESC: Leaving  main loop\r\n");
   /*
      Tell task1 and task2 to exit their while loop
   */
   task1run = task2run = 0;

   /*
      wakeup task1 and 2 from message pending, by sending two messages,
      this is necessary, because we cannot delete a message exchange, if
      any task waits on it
   */
   result = RTX_Send_Msg(create_msg.msgID,0,msg_sendbuf); //highest prio
   result = RTX_Send_Msg(create_msg.msgID,0,msg_sendbuf); //highest prio

   //wait a litte for task1 and task2
   RTX_Sleep_Time(2000);

   printf("\r\nDeleting the semaphore ");
   result = RTX_Delete_Sem(print_semaphoreID);
   if(result!=0)
   {
      printf(" failed, %d\r\n",result);
   }

   printf("\r\nDeleting the message exchange ");
   result = RTX_Delete_Msg(create_msg.msgID);

   if(result!=0)
   {
      printf(" failed, %d\r\n",result);
   }

   printf("\r\nExit program\r\n\r\n");

   //switch stdio to command shell
   BIOS_Set_Focus( FOCUS_BOTH );
   return 0;
}



// End of file
