/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     secure.c
* Function:   This example program demonstrates the usage of the IPC@Chip's
*             CGI-Interface. It installs a CGI function that can only be accessed
*             through a webbrowser if the user enters a valid combination of
*             username and password. After about two minutes the program
*             uninstalls the CGI function and exits. A browser request will now
*             return with a messae saying "Object not found".*
* Notes:      The URL of the CGI function is http://<ip-address>/secure
*             Username and password stay valid as long as the browser window is
*             not closed. So if the user accesses the page again in the same
*             window the login information won't be promted again.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\cgi\SECURE\SECURE.C, 3, 31.03.2004 11:15:57, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"


/******************************************************************************
* Definition of constants
******************************************************************************/
#define PAGENAME  "secure"    // Name of CGI function (for browser request)
#define RUNTIME   120         // Time in seconds for which the program will run
#define USERNAME  "user"      // Login information for secure page
#define PASSWORD  "password"



/******************************************************************************
* cgiFunction()
*
* This function will be executed when a browser requests the page named "secure".
* It check username and password and returns a HTML page with a resp. message
* to the webserver.
******************************************************************************/
void huge _pascal cgiFunction( rpCgiPtr CgiRequest )
{
  static char buffer[512];  // Buffer to contain web page


  // Check username and password
  if( ( strcmp( USERNAME, CgiRequest->fUserNamePtr ) != 0 ) ||
      ( strcmp( PASSWORD, CgiRequest->fPasswordPtr ) != 0 ) )
  {
    // Invalid username and/ or password
    CgiRequest->fHttpResponse = CgiHttpUnauthorized;

    // Print realm
    strcpy( buffer, "Secure realm" );
  }
  else
  {
    // Valid username and password
    CgiRequest->fHttpResponse = CgiHttpOk;

    // Construct page
    strcpy( buffer, "<html>\r\n"
                    "<head><title>Secure</title></head>\r\n"
                    "<body>\r\n"
                    "<h1>Secure - IPC@Chip CGI example</h1>\r\n"
                    "<p>Congratulation! You entered valid login information.</p>\r\n"
                    "<hr size=\"0\" />\r\n"
                    "<p><small><em>&copy; 2004 by BECK IPC GmbH</em></small></p>\r\n"
                    "</body>\r\n"
                    "</html>\r\n" );
  }

  // Give page to webserver
  CgiRequest->fResponseBufferPtr = buffer;
  CgiRequest->fResponseBufferLength = strlen( buffer );
}



/******************************************************************************
* main()
*
* Installs CGI function, runs for about 2 minutes and deinstalls CGI function
* again.
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  char *pageName = PAGENAME;  // Name of CGI function (for browser request)
  CGI_Entry cgiEntry;         // Holds information on the CGI function
  unsigned int i;             // Counter variable


  // Say hello
  printf( "\r\nSecure - IPC@Chip CGI example"
           "\r\n" );


  // Install CGI function
  cgiEntry.PathPtr = pageName;        // Name of the page
  cgiEntry.method = CgiHttpGet;       // HTTP method
  cgiEntry.CgiFuncPtr = cgiFunction;  // Function called on browser request

  if( CGI_Install( &cgiEntry ) != 0 )
  {
    printf( "\r\nInstalling CGI function %s failed"
            "\r\n", cgiEntry.PathPtr );
    return 1;
  }


  // Main loop: Increment a counter
  for( i = 0; i < 120; ++i )
  {
    // Sleep a second
    RTX_Sleep_Time( 1000 );
  }


  // Remove CGI function
  if( CGI_Delete( cgiEntry.PathPtr ) != 0 )
  {

    printf( "\r\nRemoving %s failed"
            "\r\n", cgiEntry.PathPtr );
    return 1;
  }


  // Say bye
  printf( "\r\nGood bye!" );

  return 0;
}



// End of file

