/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     tcpclie.c
* Function:   This program demonstrates the usage of the TCP/IP-API of the IPC@Chip.
*             It sends some data to an echo-server and prints out what it gets back.
* Notes:      Use this in combination with udpserv.
*             udpclie expects the IP-address of the host with the echo-server as
*             command line paraeter.
*
*******************************************************************************
*
* $Header: UDPCLIE.C, 4, 31.03.2004 11:16:08, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define TM_PORT_ECHO                      7    /*well known echo port*/
#define TM_UDP_CLIENT_ECHOPORT         3210
#define TM_UDPECHOBUF_CLIENT_SENDSIZE    25
#define TM_UDPECHOBUF_CLIENT_RECVSIZE    25

/*
  this switch is only for debugging:
  comment this switch, if you haved finished your application
  because the print routines takes very much processing time
*/
//#define UDP_ECHO_CLIENT_DEBUG



/******************************************************************************
* Global variables
******************************************************************************/
static unsigned long recv_errors=0L;
static unsigned long send_errors=0L;
static unsigned long miss_errors=0L;
static unsigned long dupl_errors=0L;

static char sendbuf[TM_UDPECHOBUF_CLIENT_SENDSIZE ]; //only for udpechoclient
static char recvbuf[TM_UDPECHOBUF_CLIENT_RECVSIZE ]; //only for udpechoclient

static char  destIPStr[17];

unsigned char last_blk;



/******************************************************************************
* main()
******************************************************************************/
int main(int argc, char * argv[])
{
   struct sockaddr_in   addr;
   struct sockaddr_in   raddr;

   int error;
   int result;
   int sd;
   int i;
   int j=0;

   
   strcpy(destIPStr,"192.168.200.1");

   if(argc==2)
   {
    if(strlen(argv[1]) < 17)
      strcpy(destIPStr,argv[1]);
   }
   else
   {
     printf("\r\nUDPClient: Missing or invalid argument\r\n");

   }
   

   //set stdio focus to user
   BIOS_Set_Focus( FOCUS_APPLICATION );

   printf("\r\nUDPEchoClient:\r\nDest. IP: %s ,  Port %d\r\n",destIPStr,TM_PORT_ECHO);
   #ifdef UDP_ECHO_CLIENT_DEBUG
   printf("\r\nOpen socket");
   #endif

   //API call open socket
   sd = opensocket( SOCK_DGRAM, &error );

   if(error != 0)
   {
    printf("\r\nSocket open failed: %d",error);
    goto UDP_CLIENT_DONE;
   }

   //init the sendbuffer
   sendbuf[0]= 0;
   for(i=1;i<TM_UDPECHOBUF_CLIENT_SENDSIZE;i++)
   {
      sendbuf[i]='a';
   }

   while(1)   //loop forever
   {
     /**************************************************************/
     //send some data to specified echoserver
     /**************************************************************/
     addr.sin_family      = AF_INET;

     //set destip, using inetaddr
     if( inet_addr( destIPStr, &addr.sin_addr.s_addr ) != 0 )
     {
       printf("\r\nCannot convert %s  ",destIPStr);
       goto UDP_CLIENT_DONE;
     }

     //set dest.port, using htons
     addr.sin_port = htons( TM_PORT_ECHO );

     //API-Call sendto
     if( sendto( sd, sendbuf, TM_UDPECHOBUF_CLIENT_SENDSIZE, 0,
         &addr, &error ) == API_ERROR )
     {
       #ifdef UDP_ECHO_CLIENT_DEBUG
       printf("\r\nSenderror %d",error);
       #endif
       send_errors++;
       //goto UDP_CLIENT_DONE;
     }

     printf("\r\nSending  Blk   %5u Chars: ",sendbuf[0]&0x00FF);
     for(i=1;i<TM_UDPECHOBUF_CLIENT_SENDSIZE;i++)
     {
      printf("%c",(char)sendbuf[i]);
     }

     /*****************************************************************/
     //Sleep a little bit
     /*****************************************************************/
     RTX_Sleep_Time( 500 );

     /*******************************************************************/
     //Wait for an answer from the specified echoserver, call recvfrom
     //To prevent a socketoverrun, read all available data from the socket
     /*******************************************************************/

     do
     {
      if( ( result = recvfrom( sd, recvbuf, TM_UDPECHOBUF_CLIENT_RECVSIZE,
            MSG_TIMEOUT, 10000L, &raddr, &error ) ) == API_ERROR )
      {
         #ifdef UDP_ECHO_CLIENT_DEBUG
         printf("\r\nRecverror %d",outregs.x.ax);
         #endif
         recv_errors++;
      }
      else
      {
         if(result>0)    //data received
         {
          //Check blknumber of the received packet
          if(last_blk == recvbuf[0])
          {
            printf("\r\nDuplicated packet: %5u",recvbuf[0]&0x00FF);
            dupl_errors++;
          }
          else
          {
            //Is it the last sent packet?
            if(sendbuf[0]==recvbuf[0])
            {
             printf("\r\nGot last echo: %5u Chars: ",recvbuf[0]&0x00FF);
             for(i=1;i<result;i++)
             {
               printf("%c",(char)recvbuf[i]);
             }//for(i=0;i<result;i++)
             last_blk = recvbuf[0];
            }
            else
            {
             miss_errors++;
            }
          }
         }    //data received
         else
         {
          recv_errors++;
         }
      }
   }
   //while data available
   while( GetWaitingBytes( sd, &error ) > 0 );

     //modify the sendbuffer for the next sendjob
     for(i=1;i<TM_UDPECHOBUF_CLIENT_SENDSIZE;i++)
     {
       if(sendbuf[i]<'z')
       {
         sendbuf[i]++;
       }
       else
       {
         sendbuf[i]='a';
       }
     }
     sendbuf[0]++;

     /*show error statistic*/
     if(j++==50)
     {
      j=0;
      printf("\r\nUDPEchoClient: SErrs %8lu, RErrs: %8lu, Missed %8lu Dupl. %8lu\r\n"
            ,send_errors,recv_errors ,miss_errors, dupl_errors);
     }
   }//while(1)   //loop forever


   //should not happen
UDP_CLIENT_DONE:
   //close socket
   if( closesocket( sd, &error ) == API_ERROR )
   {
    printf("\r\nSocket close failed: %d",error);
   }

   //set stdio focus to user
   //set stdio focus to shell
   BIOS_Set_Focus( FOCUS_BOTH );

   return 0; 
}
/*************************************************************************/
//end udpclie.c
/*************************************************************************/
