/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     dkxxcgi.c
* Function:   This example program demonstrates the usage of the IPC@Chip's
*             CGI-Interface. It installs a CGI function that presents - if called
*             without arguments - a HTML page which shows the actual values of the
*             inputs, output and the PIOs 3 and 13 of the DK50 and some links to
*             change these values. When called with arguments the function sets
*             I/Os and/ or PIOs according to the arguments.
* Note:       The URL of the CGI function is http://<ip-address>/dkxx
*             To increase speed in your own CGI functions the resulting HTML pages
*             should be much smaller than the one produced here.
*             On a DK40 pin 7 of the terminal strip is not bit 7 of the I/Os
*             but PIO13.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\cgi\dkxxcgi\dkxxcgi.c, 2, 31.03.2004 11:15:37, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"



/******************************************************************************
* Defines
******************************************************************************/
#define VERSION  0x0004  // Program version
#define YEAR     "2004"  // Year for copyritgh information
#define URL      "dkxx"  // URL for CGI function

// Debug
//#define DEBUG



/******************************************************************************
* Global variables
******************************************************************************/
char *url = URL;               // URL for CGI function
unsigned int outputValue = 0,  // Save value of outputs
             pio03 = 0,        // Save value of PIO3
             pio13 = 0;        // Save value of PIO13



/******************************************************************************
* CGI function
******************************************************************************/
void huge _pascal cgiFunction( rpCgiPtr request )
{
  unsigned int inputValue = 0,  // Actual value of inputs
               i = 0,           // Counter variable
               tmp = 0;         // Temporary
  static char htmlPage[2048];   // Buffer to store HTML page in
  char tmpStr[50];              // Temporary


  #ifdef DEBUG
  printf( "\r\nDKxxCGI Request:" );
  #endif

  // If there are arguments, parse them
  if( request->fArgumentBufferLength != 0l )
  {
    #ifdef DEBUG
    printf( " %s", request->fArgumentBufferPtr );
    #endif

    // Parse outputs
    if( sscanf( request->fArgumentBufferPtr, "O=0x%02X", &tmp ) )
    {
      outputValue = tmp;
    }

    for( i = 0; i < 8; ++i )
    {
      sprintf( tmpStr, "O%i=%%i", i );
      if( sscanf( request->fArgumentBufferPtr, tmpStr, &tmp ) )
      {
        outputValue = ( tmp ) ? outputValue | ( 1 << i )
                              : outputValue & ~( 1 << i );
      }
    }

    // Parse PIOs
    if( sscanf( request->fArgumentBufferPtr, "PIO03=%i", &tmp ) )
    {
      pio03 = ( tmp ) ? 1 : 0;
    }

    if( sscanf( request->fArgumentBufferPtr, "PIO13=%i", &tmp ) )
    {
      pio13 = ( tmp ) ? 1 : 0;
    }

    // Put out
    hal_write_bus( 0x0600, outputValue, 0xFFFF, 0x0000 );  // Output
    hal_write_pio( 3, ( pio03 ) ? 1 : 0 );  // PIO3
    hal_write_pio( 13, ( pio13 ) ? 1 : 0 );  // PI13
  }


  // Send page
  // Get actual input value
  inputValue = hal_read_bus( 0x0600, 0xFFFF, 0x0000 );
                  
  // Build HTML page
  strcpy( htmlPage, "<html>\r\n"
                    "<head>\r\n"
                    "<title>DKxxCGI - IPC@CHIP CGI example</title>\r\n"
                    "<link rel=\"stylesheet\" type=\"text/css\" href=\"dkxx.css\" />\r\n"
                    "</head>\r\n"
                    "<body>\r\n"
                    "<b>DKxxCGI - IPC@CHIP CGI example</b>\r\n"
                    "<hr size=\"0\" />\r\n"
                    "<table cellpadding=\"16\"><tr valign=\"top\">\r\n" );

  // Print Inputs
  strcat( htmlPage, "<td>\r\n"
                    "<table>\r\n"
                    "<tr valign=\"top\">\r\n"
                    "<th>Inputs</th>\r\n"
                    "</tr>\r\n"
                    "<tr valign=\"top\">\r\n"
                    "<td>\r\n" );
  sprintf( tmpStr, "I=0x%02X<br>\r\n", inputValue & 0x00FF );
  strcat( htmlPage, tmpStr );
  for( i = 0; i < 8; ++i )
  {
    sprintf( tmpStr, "I%i=%i<br>\r\n",
                  i, ( inputValue >> i) & 0x0001 );
    strcat( htmlPage, tmpStr );
  }
  strcat( htmlPage, "</td>\r\n"
                    "</tr>\r\n"
                    "</table>\r\n"
                    "</td>\r\n" );

  // Print Outputs
  strcat( htmlPage, "<td>\r\n"
                    "<table>\r\n"
                    "<tr valign=\"top\">\r\n"
                    "<th>Outputs</th>\r\n"
                    "</tr>\r\n"
                    "<tr valign=\"top\">\r\n"
                    "<td nowrap>\r\n" );
  sprintf( tmpStr, "O=0x%02X<br>\r\n", outputValue & 0x00FF );
  strcat( htmlPage, tmpStr );
  for( i = 0; i < 8; ++i )
  {
    sprintf( tmpStr, "O%i=%i", i, ( outputValue >> i) & 0x0001 );
    strcat( htmlPage, tmpStr );
    sprintf( tmpStr, " <a href=\"%s?O%i=1\">Set</a>", url, i );
    strcat( htmlPage, tmpStr );
    sprintf( tmpStr, " <a href=\"%s?O%i=0\">Reset</a><br>\r\n", url, i );
    strcat( htmlPage, tmpStr );
  }
  strcat( htmlPage, "</td>\r\n"
                    "</tr>\r\n"
                    "</table>\r\n"
                    "</td>\r\n" );

  // Print PIOs
  strcat( htmlPage, "<td>\r\n"
                    "<table>\r\n"
                    "<tr valign=\"top\">\r\n"
                    "<th>PIOs</th>\r\n"
                    "</tr>\r\n"
                    "<tr valign=\"top\">\r\n"
                    "<td>\r\n" );
  sprintf( tmpStr, "PIO03=%i", ( pio03 ) ? 1 : 0 );
  strcat( htmlPage, tmpStr );
  sprintf( tmpStr, " <a href=\"%s?PIO03=1\">Set</a>", url );
  strcat( htmlPage, tmpStr );
  sprintf( tmpStr, " <a href=\"%s?PIO03=0\">Reset</a><br>\r\n", url );
  strcat( htmlPage, tmpStr );
  sprintf( tmpStr, "PIO13=%i", ( pio13 ) ? 1 : 0 );
  strcat( htmlPage, tmpStr );
  sprintf( tmpStr, " <a href=\"%s?PIO13=1\">Set</a>", url );
  strcat( htmlPage, tmpStr );
  sprintf( tmpStr, " <a href=\"%s?PIO13=0\">Reset</a><br>\r\n", url );
  strcat( htmlPage, tmpStr );
  strcat( htmlPage, "Note: The LED PIO3 is on, if PIO3 has value 0.<br>" );
  strcat( htmlPage, "</td>\r\n"
                    "</tr>\r\n"
                    "</table>\r\n"
                    "</td>\r\n"
                    "</tr></table>\r\n" );

  // Print Flash parameter string
  strcat( htmlPage, "<!-- Flash: " );
  for( i = 0; i < 8; ++i )
  {
    sprintf( tmpStr, "&I%i=%i", i, ( inputValue >> i ) & 0x0001 );
    strcat( htmlPage, tmpStr );
    sprintf( tmpStr, "&O%i=%i", i, ( outputValue >> i ) & 0x0001 );
    strcat( htmlPage, tmpStr );
  }
  sprintf( tmpStr, "&PIO03=%i", ( pio03 ) ? 1 : 0 );
  strcat( htmlPage, tmpStr );
  sprintf( tmpStr, "&PIO13=%i", ( pio13 ) ? 1 : 0 );
  strcat( htmlPage, tmpStr );
  strcat( htmlPage, "&datas=true& -->\r\n" );

  // Print HTML tail
  strcat( htmlPage, "<hr size=\"0\" />\r\n"
                    "<p align=\"center\">\r\n"
                    "<small><em>&copy; "YEAR" Beck IPC GmbH</em></small>\r\n"
                    "</p>\r\n"
                    "</body>\r\n"
                    "</html>\r\n" );

  // Send HTML page
  request->fHttpResponse = CgiHttpOk;
  request->fResponseBufferPtr = htmlPage;
  request->fResponseBufferLength = strlen( htmlPage );

  return;
}



/******************************************************************************
* Main function
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  CGI_Entry cgi;  // Structure to store CGI entry in


  // Say hello
  printf( "\r\nDKxxCGI V%X.%02X\r\n", ( VERSION >> 16 ) & 0x00FF,
                              VERSION & 0x00FF );


  // Enable I/Os by setting PCS6
  pfe_enable_pcs( 6 );

  // Enable ALE
  pfe_enable_bus( 0x00FF, 1 );

  // Enable PIO3 and PIO13
  pfe_enable_pio( 3, 5 );
  pfe_enable_pio( 13, 5 );


  // Set all outputs to 0
  hal_write_bus( 0x0600, 0, 0xFFFF, 0x0000 );

  // Set PIOs to 0
  hal_write_pio( 3, 0 );
  hal_write_pio( 13, 0 );


  // Install CGI function
  cgi.PathPtr = url;            // URL
  cgi.method = CgiHttpGet;       // HTTP method
  cgi.CgiFuncPtr = cgiFunction;  // CGI function

  if( CGI_Install( &cgi ) != 0 )
  {
    // Error
    printf( "\r\nInstallation of CGI function %s failed."
            "\r\n", url );
    return 1;
  }
  else
  {
    // OK
    printf( "\r\nInstallation of CGI function %s successfully completed."
            "\r\n", url );
  }


  // Stay resident
  while( 1 )
  {
    // Sleep a second
    RTX_Sleep_Time( 1000 );
  }
}



// End of file

