/******************************************************************************
*
* (c) 2003-2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
* Module:     ftpc.c
* Function:   This is a very simple FTP client.
*             The module ftplib.c is used which implements the most important
*             parts of the File Transfer Protocol in easy to use C-functions.
*             If you want to use FTP in your own applications you may include
*             ftplib.c in your project and use this file as an example of how
*             to use it.
* Notes:      To get a help message for this program start it with option -h.
*
*******************************************************************************
*
* $Header: ftpc.c, 6, 11.05.2004 12:19:44, Jan Schatz$
*
******************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <clib.h>
#include "ftplib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define VERSION  0x0112


/******************************************************************************
* usage()
*
* Prints a usage message
******************************************************************************/
void usage(
            void )
{
  printf( "\r\nUsage: FTPC [OPTIONS] IP USER PASSWORD ACTION SOURCE [DESTINATION]"
          "\r\n  OPTIONS      One of the following"
          "\r\n               -H  Print this help message and exit"
          "\r\n               -P  Use passive mode for transfer"
          "\r\n               -A  Use ASCII representation for transfer (Default is binary)"
          "\r\n  IP           IP address of FTP server"
          "\r\n  USER"
          "\r\n  PASSWORD     Login information for FTP account"
          "\r\n  ACTION       One of the following:"
          "\r\n               GET  The remote file specified in SOURCE will be copied to the"
          "\r\n                    local file specified in DESTINATION."
          "\r\n               PUT  The local file specified in SOURCE will be copied to the"
          "\r\n                    remote file specified in DESTINATION."
          "\r\n               DEL  The file specified in SOURCE will be deleted."
          "\r\n               REN  The file/directory specified in SOURCE will be given the"
          "\r\n                    name specified in DESTINATION."
          "\r\n               DIR  The content of the directory specified in SOURCE will be"
          "\r\n                    displayed."
          "\r\n               MKD  The directory specified in SOURCE will be created."
          "\r\n               RMD  The directory specified in SOURCE will be deleted."
          "\r\n  SOURCE"
          "\r\n  DESTINATION  See description for ACTION." );
}



/******************************************************************************
* quit()
*
* Quits the program cleanly
******************************************************************************/
void quit(
           int result )
{
  // Say bye
  printf( "\r\n"
          "\r\nGood Bye."
          "\r\n" );

  // Set STDIO focus to both
  BIOS_Set_Focus( FOCUS_BOTH );

  // Return the given result to environment
  exit( result );
}



/******************************************************************************
* main()
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  int controlSocket,                      // Socket descriptor of control connection
      i;                                  // Counter variable
  char *remoteIp = NULL,                  // IP address of remote system
       *user = NULL,
       *password = NULL,                  // Login information
       *source = NULL,                    // Source file or directory
       *destination = NULL;               // Destination file
  enum {
         ARGUMENT_NO,
         ARGUMENT_IP,
         ARGUMENT_USER,
         ARGUMENT_PASSWORD,
         ARGUMENT_ACTION,
         ARGUMENT_SOURCE,
         ARGUMENT_DESTINATION
       } expectedArgument = ARGUMENT_IP;  // Next command line argument expected
  enum {
         ACTION_UNKNOWN,
         ACTION_GET,
         ACTION_PUT,
         ACTION_DEL,
         ACTION_REN,
         ACTION_DIR,
         ACTION_MKD,
         ACTION_RMD,
       } action = ACTION_UNKNOWN;         // FTP action to be taken
  BOOL usePassiveMode = FALSE,            // Will be set TRUE if passive mode is
                                          // to be used
       useAsciiRepresentation = FALSE;    // Will be set TRUE if ASCII representation
                                          // is to be used


  // Set STDIO focus to application
  BIOS_Set_Focus( FOCUS_APPLICATION );

  // Say hello
  printf( "\r\n--------------------------------------------------"
          "\r\nIPC@Chip FTP-Client V %X.%02X"
          "\r\n--------------------------------------------------"
          "\r\n", VERSION >> 8, VERSION & 0xFF );


  // Parse command line arguments
  for( i = 1; i < argc; ++i )
  {
    // Option found
    if( argv[i][0] == '-' )
    {
      if( stricmp( "-H", argv[i] ) == 0 )
      {
        usage();
        quit( 0 );
      }
      if( stricmp( "-P", argv[i] ) == 0 )
        usePassiveMode = TRUE;
      if( stricmp( "-A", argv[i] ) == 0 )
        useAsciiRepresentation = TRUE;

      // Don't interpret option as any other argument
      continue;
    }

    // Other argument
    switch( expectedArgument )
    {
      case ARGUMENT_IP:  // IP argument expected
        // Save argument and get next one
        remoteIp = argv[i];
        expectedArgument = ARGUMENT_USER;
        break;

      case ARGUMENT_USER:  // PASSWORD argument expected
        // Save argument and get next one
        user = argv[i];
        expectedArgument = ARGUMENT_PASSWORD;
        break;

      case ARGUMENT_PASSWORD:  // PASSWORD argument expected
        // Save argument and get next one
        password = argv[i];
        expectedArgument = ARGUMENT_ACTION;
        break;

      case ARGUMENT_ACTION:  // ACTION argument expected
        // Check which action to take
        if( stricmp( "GET", argv[i] ) == 0 )
        {
          action = ACTION_GET;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        if( stricmp( "PUT", argv[i] ) == 0 )
        {
          action = ACTION_PUT;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        if( stricmp( "DEL", argv[i] ) == 0 )
        {
          action = ACTION_DEL;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        if( stricmp( "REN", argv[i] ) == 0 )
        {
          action = ACTION_REN;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        if( stricmp( "DIR", argv[i] ) == 0 )
        {
          action = ACTION_DIR;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        if( stricmp( "MKD", argv[i] ) == 0 )
        {
          action = ACTION_MKD;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        if( stricmp( "RMD", argv[i] ) == 0 )
        {
          action = ACTION_RMD;
          expectedArgument = ARGUMENT_SOURCE;
          break;
        }
        // else
        printf( "\r\nError: No or invalid action specified." );
        usage();
        quit( EXIT_FAILURE );

      case ARGUMENT_SOURCE:  // SOURCE argument expected
        // Save argument and get next one if needed
        source = argv[i];
        switch( action )
        {
          case ACTION_GET:
          case ACTION_PUT:
          case ACTION_REN:
            expectedArgument = ARGUMENT_DESTINATION;  // Destination needed
            break;

          case ACTION_DEL:
          case ACTION_DIR:
          case ACTION_MKD:
          case ACTION_RMD:
            expectedArgument = ARGUMENT_NO;  // No more arguments needed
            break;

          default:  // Error
            printf( "\r\nError: No or invalid action specified." );
            usage();
            quit( EXIT_FAILURE );
        }
        break;

      case ARGUMENT_DESTINATION:  // DESTINATION argument expected
        // Save argument and get next one
        destination = argv[i];
        expectedArgument = ARGUMENT_NO;
        break;
    }  // switch( expectedArgument )
  }  // for( i=...

  // Check command line arguments
  // Check IP argument
  if( remoteIp == NULL )
  {
    printf( "\r\nError: IP argument not specified." );
    usage();
    quit( EXIT_FAILURE );
  }

  // Check USER argument
  if( user == NULL )
  {
    printf( "\r\nError: USER argument not specified." );
    usage();
    quit( EXIT_FAILURE );
  }

  // Check PASSWORD argument
  if( password == NULL )
  {
    printf( "\r\nError: PASSWORD argument not specified." );
    usage();
    quit( EXIT_FAILURE );
  }

  // Check ACTION argument
  if( action == ACTION_UNKNOWN )
  {
    printf( "\r\nError: No or invalid action specified." );
    usage();
    quit( EXIT_FAILURE );
  }

  // Check SOURCE argument
  if( source == NULL )
  {
    printf( "\r\nError: SOURCE argument not specified." );
    usage();
    quit( EXIT_FAILURE );
  }

  // Check DESTINATION argument
  if( ( action == ACTION_GET || action == ACTION_PUT || action == ACTION_REN ) &&
      destination == NULL )
  {
    printf( "\r\nError: DESTINATION argument not specified." );
    usage();
    quit( EXIT_FAILURE );
  }


  // Open control connection
  if( ( controlSocket = ftpOpenControlConnection( remoteIp ) ) == -1 )
  {
    printf( "\r\nError: Could not connect to %s.", remoteIp );
    quit( EXIT_FAILURE );
  }

  // Login
  if( !ftpLogin( controlSocket, user, password, NULL ) )
  {
    printf( "\r\nError: Login failed for user %s.", user );
    quit( EXIT_FAILURE );
  }

  // Take specified action
  switch( action )
  {
    case ACTION_GET:  // Get file from remote host
      printf( "\r\nGetting file %s ...", source );
      if( !ftpReceiveFile( controlSocket, destination, source,
                           ( useAsciiRepresentation ) ? FTP_TYPE_ASCII_NONPRINT : FTP_TYPE_IMAGE,
                           usePassiveMode ) )
        printf( "\r\nError: File %s could not be gotten.", source );
      else
        printf( "\r\nDone." );
      break;

    case ACTION_PUT:  // Put file to remote host
      printf( "\r\nPutting file %s ...", source );
      if( !ftpSendFile( controlSocket, destination, source,
                        ( useAsciiRepresentation ) ? FTP_TYPE_ASCII_NONPRINT : FTP_TYPE_IMAGE,
                        usePassiveMode ) )
        printf( "\r\nError: File %s could not be put.", source );
      else
        printf( "\r\nDone." );
      break;

    case ACTION_DEL:  // Delete file in source
      if( !ftpDelete( controlSocket, source ) )
        printf( "\r\nError: Could not delete file %s.", source );
      else
        printf( "\r\nFile %s successfully deleted.", source );
      break;

    case ACTION_REN:  // Rename file/directory in source to destination
      if( !ftpRename( controlSocket, destination, source ) )
        printf( "\r\nError: Could not rename file/directory %s to %s.", source,
                                                                        destination );
      else
        printf( "\r\nFile/directory %s successfully renamed to %s.", source,
                                                                     destination );
      break;

    case ACTION_DIR:  // List directory in source
      if( !ftpSetDirectory( controlSocket, source ) ||
          !ftpListDirectory( controlSocket, "", stdout, usePassiveMode ) )
        printf( "\r\nError: Could not list directory %s.", source );
      break;

    case ACTION_MKD:  // Create directory in source
      if( !ftpCreateDirectory( controlSocket, source ) )
        printf( "\r\nError: Could not create directory %s.", source );
      else
        printf( "\r\nDirectory %s successfully created.", source );
      break;

    case ACTION_RMD:  // Delete directory in source
      if( !ftpDeleteDirectory( controlSocket, source ) )
        printf( "\r\nError: Could not delete directory %s.", source );
      else
        printf( "\r\nDirectory %s successfully deleted.", source );
      break;
  }

  // Logout
  if( !ftpLogout( controlSocket ) )
    printf( "\r\nWarning: Logout failed." );


  // Say bye
  quit( EXIT_SUCCESS ); 
  return 0;  // Will not be reached
}



// End of file

