/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     piopin.c
* Function:   Example for using the hardware API. This shows how to enable the
*             programmable IO pins.
*             This example provides almost the same functionality as the
*             internal "pio" RTOS command.
*
*******************************************************************************
*
* $Header: piopin.c, 2, 31.03.2004 11:15:51, Andre Pribil$
*
******************************************************************************/



/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Display command line options
******************************************************************************/
void usage (void)
{
  printf("\nProgram shows and modifies PIO state");
  printf("\nUsage: piopin <pio-no> [mode],  pio-no=0..13 , mode=1..5");
  printf("\n Mode: 1 = Input without pullup/down");
  printf("\n       2 = Input with pullup (not PIO13)");
  printf("\n       3 = Input with pulldown (only PIO3 and PIO13)");
  printf("\n       4 = Output (Init value = High)");
  printf("\n       5 = Output (Init value = Low)");
}

/******************************************************************************
* Main
******************************************************************************/
void main (int argc, char *argv[])
{
  unsigned char pio;
  unsigned char mode;

  unsigned wState, wPio, wInp, wOut;

  // set mode
  if (argc==3)
  {
     pio = atoi(argv[1]);
     if (pio>13)
     {
       usage();
       return;
     }

     mode = atoi(argv[2]);
     if ((mode==0) || (mode>5))
     {
       usage();
       return;
     }

     printf("\nSet mode=%d for PIO%d", mode, pio);

     // enable pio pins
     pfe_enable_pio(pio,mode);
  }
  // read state
  else if (argc==2)
  {
     pio = atoi(argv[1]);
     if (pio>13)
     {
       usage();
       return;
     }

     // read pio states
     pfe_get_pio_state(&wPio,&wInp,&wOut);

     // read pio pin
     wState = hal_read_pio (pio);

     printf("\r\nPIO%u is %s pin, pin is %s, value=%s",
            pio,
            wPio&(1<<pio) ? "a pio" : "not a pio",
            wInp&(1<<pio) ? "input" : "output",
            wState ? "1" : "0");
  }
  else
  {
    usage();
    return;
  }
}
