/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     powsav.c
* Function:   Example demonstrates the usage of the SC13 powersave modes.
*
*******************************************************************************
*
* $Header: powsav.c, 2, 31.03.2004 11:15:52, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include "clib.h"

/******************************************************************************
* Variables
******************************************************************************/
volatile unsigned int interrupt_done = 0;


/******************************************************************************
* Interrupt handler
******************************************************************************/
void huge int_handler (void)
{
    // Call BIOSINT Power Save, Leave sleep mode
    BIOS_Power_SaveEx (0x81);

    interrupt_done = 1;
}

/******************************************************************************
* Display command line parameters
******************************************************************************/
void usage (void)
{
    printf("\nUsage: powsav.exe <mode> (Only for SC11/SC13)"
           "\n       mode = 1,   Full clock speed"
           "\n       mode = 2,   1/2 clock speed"
           "\n       mode = 4,   1/4 clock speed"
           "\n       mode = 128, Sleep mode, wakeup through INT0"
           "\n       mode = 32,  Power down ethernet"
           "\n       mode = 33,  Power up ethernet");
}


/******************************************************************************
* Main
******************************************************************************/
void main (int argc, char *argv[])
{
  unsigned char mode;

  if (argc < 2)
  {
    usage();
    return;
  }

  mode = atoi(argv[1]);
  switch (mode)
  {
    case 1:   printf("\nSet full speed mode");
              break;
    case 2:   printf("\nSet 1/2 speed mode");
              break;
    case 4:   printf("\nSet 1/4 speed mode");
              break;
    case 128: printf("\nSet sleep mode, trigger INT0 for wakeup");
              break;
    case 32:  printf("\nPower down ethernet");
              break;
    case 33:  printf("\nPower up ethernet");
              break;
    default:  usage();
              return;
  }

  // Sleep
  RTX_Sleep_Time(50);

  // If not sleep mode, set mode and exit
  if (mode != 128)
  {
    // Call BIOSINT Power save
    BIOS_Power_SaveEx (mode);

    printf("\nDone\n");
  }
  else // Sleep mode, install INT0 handler
  {
    // Enable external interrupt requests
    pfe_enable_int(0);

    // Set service interrupt routine
    hal_install_isr (0, 1, int_handler);

    // Call BIOSINT Power save, Enter sleep mode
    BIOS_Power_SaveEx (0x80);

    while (!interrupt_done)
    {
      // do nothing ...
    }

    // Set service interrupt routine
    hal_install_isr (0, 0, NULL);

    printf("\nWaked up by INT0"
           "\nDone");
  }
}
