/****************************************************************************
*
* (C) 2004 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-240
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module      :  mib.h
                 Requires SC13 @CHIP-RTOS V0.91 or SC12 @CHIP-RTOS V1.10 or higher
* Function    :  Constants/Enums, types for MIB definition
*******************************************************************************
*
* $Header: mib.h, 1, 15.03.2004 10:29:18, Markus Bartat$
*
******************************************************************************
* THIS SOFTWARE IS PROVIDED BY BECK IPC GmbH ``AS IS'' AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* ******************************************************************************
*/

/******************************************************************************/
#ifndef    _MIB_H_
/******************************************************************************/
   #define _MIB_H_

   #include <typedef.h>
   #include <snmphlp.h>

   #define  READ                       0
   #define  READ_WRITE                 1
   #define  TRAP_ENABLED               1
   #define  TRAP_DISABLED              2
   #define  MIB_ADDRESS_LENGTH       128
   #define  BASE_OID_STRLEN           13   //min length of a BaseOID string

   //mib definition structure, used at mib.c
   typedef struct tMib
   {
      unsigned int  baseOIDindex;  // base oid string
      char          *abAddress;    // oid string, appended at base oid string
      unsigned char bAccess;       // access type (READ, READ_WRITE,...)
      unsigned char bSyntax;       // data type (SNMP_STRING,...)
      unsigned int  wChannel;      // index for finding mib entry, enum MIB_II below
      unsigned char tbl_idx;       // index number used for dynamic table members
      struct tMib * mibPointer;    // pointer to next entry of a dynamic sub mib table
                                   // or entry of main mib (tMIB),
                                   // if a null pointer is used, the next entry is
                                   // sucessor of the static main mib table tMIB
   }tMib;


   //index number for MIB-II entries
   enum MIB_II
       {  sysDescr, sysObjectID, sysUpTime, sysContact, sysName, sysLocation, sysService,
          ifNumber, ifIndex, ifDescr, ifType, ifMtu, ifSpeed, ifPhysAddress, ifAdminStatus,
          ifOperStatus, ifLastChange, ifInOctets, ifInUcastPkts, ifInNucastPkts, ifInDiscards,
          ifInErrors, ifInUnknownProtos, ifOutOctets, ifOutUcastPkts,ifOutNUcastPkts, ifOutDicards, ifOutErrors,
          ifOutLen, ifSpecific,
          atIfIndex, atPhysAddress, atNetAddress,
          ipForwarding, ipDefaultTTL, ipInReceives, ipInHdrErrors, ipInAddrErrors, ipForwDatagrams,
          ipInUnknownProtos, ipInDiscards, ipInDelivers, ipOutrequests, ipOutDiscards, ipOutNoRoutes,
          ipreasmTimeout, ipReasmRedqs, ipReasmOKs, ipReasmFails, ipFragOKs, ipFragFails,
          ipFragCreates, ipAdEntAddr, ipAdEntIfIndex, ipAdEntNetMask, ipAdEntBcastAddr, ipAdEntReasmMaxSize,
          ipRouteDest, ipRouteIfIndex, ipRouteMetric1, ipRouteMetric2, ipRouteMetric3, ipRouteMetric4,
          ipRouteNextHop, ipRouteType, ipRouteProto, ipRouteAge, ipRouteMask, ipRouteMetric, ipRouteInfo,
          ipNetToMediaIfIndex, ipNetToMediaPhyAddress, ipNetToMediaNetAddress, ipNetToMediaType,
          ipRoutingDiscards,
          icmpInMsgs, icmpInErrors, icmpInDestUnreachs, icmpInTimeExcds, icmpInParmProbs,
          icmpInSrcQuenchs, icmpRedirects, icmpInEchos, icmpInEchoReps, icmpInTimestamps, 
          icmpInTimestampReps, icmpInAddrMasks, icmpInAddrMaskReps, icmpOutMsgs, icmpOutErrors,
          icmpOutDestUnreachs, icmpOutTimeExcds, icmpOutParmProbs, icmpOutSrcQuenchs, icmpOutRedirects,
          icmpOutEchos, icmpOutEchoReps, icmpOutTimestamps, icmpOutTimestampReps, icmpOutAddrMasks, icmpOutAddrMaskReps,
          tcpRtoAlgorithm, tcpRtoMin, tcpRtoMax, tcpMaxConn, tcpActiveOpens, tcpPassivOpens, tcpAttemptFails,
          tcpEstabResets, tcpCurrEstab, tcpInSegs, tcpOutSegs, tcpRetransSegs,
          tcpConnState, tcpConnLocalAddress, tcpConnLocalPort, tcpConnRemAddress, tcpConnRemPort,
          tcpInErrs, tcpOutRsts,
          udpInDatagrams, udpNoPorts, udpInErrors, udpOutDatagrams, udpLocalAddress, udpLocalPort,
          snmpInPkts,snmpOutPkts ,snmpInBadVersions, snmpInBadCommunityNames, snmpInBadCommunityUses, snmpInASNParseErrs,
          snmpInTooBigs, snmpInNoSuchNames, snmpInBadValues, snmpInReadOnlys, snmpIngGenErrs,
          snmpInTotalReqVars, snmpInTotalSetVars, snmpInGetRequests, snmpInGetNexts, snmpInSetReqeusts,
          snmpInGetResponses, snmpInTraps, snmpOutTooBigs, snmpOutNoSuchNames, snmpOutBadValues,
          snmpOutGenErrs, snmpOutGetRequests, snmpOutGetNexts, snmpOutSetRequests,
          snmpOutGetResponses, snmpOutTraps, snmpEnableAuthTraps
          #ifdef DKXX_SNMP_DEMO
          ,dkxxOut0,dkxxOut1,dkxxOut2,dkxxOut3,dkxxOut4,dkxxOut5,dkxxOut6,dkxxOut7
          #endif

   };

#endif /* _MIB_H_*/
/**********************************************************************************************************************************/
//end mib.h
